#include "pstools.hpp"

#include <boost/filesystem/operations.hpp>
#include <boost/filesystem/fstream.hpp>
#include<boost/tokenizer.hpp>
#include<boost/lexical_cast.hpp>
#include <cstdlib>
#include <iostream>


namespace bfs = boost::filesystem;

namespace pstome {

PsTools::PsTools(std::string s_infile)
        : pagecount(0),is_pdf(false),psselect(""),psnup(""),pdf2ps(""),infile(s_infile)
{
        
        bfs::path path_to_infile (bfs::initial_path());
        path_to_infile=bfs::system_complete( bfs::path( infile, bfs::native ) );

        if (!bfs::exists(path_to_infile)){
                throw INFILENOTFOUND;
        }
        

        std::string PATH="";
        char * buffer;
        buffer = getenv ("PATH");
        if (buffer!=NULL){
                PATH=buffer;
        }

        if (PATH != ""){
                typedef boost::escaped_list_separator<char> t_seperator;
                t_seperator sep('\\', ':', '\"');
                boost::tokenizer<t_seperator > pathlist(PATH,sep);
                boost::tokenizer<t_seperator >::iterator pi;
                std::cout << std::endl;
                for (pi=pathlist.begin(); pi!=pathlist.end(); pi++){
                        bfs::path path_to_psselect (bfs::initial_path());
                        bfs::path path_to_psnup (bfs::initial_path());
                        bfs::path path_to_pdf2ps (bfs::initial_path());
                        path_to_psselect=bfs::system_complete( bfs::path( (*pi)+"/psselect", bfs::native ) );
                        path_to_psnup=bfs::system_complete( bfs::path( (*pi)+"/psnup", bfs::native ) );
                        path_to_pdf2ps=bfs::system_complete( bfs::path( (*pi)+"/pdf2ps", bfs::native ) );
                        if (bfs::exists(path_to_psselect)){
                                psselect=path_to_psselect.native_file_string();
                        } 
                        if (bfs::exists(path_to_psnup)){
                                psnup=path_to_psnup.native_file_string();
                        }
                        if (bfs::exists(path_to_pdf2ps)){
                                pdf2ps=path_to_pdf2ps.native_file_string();
                        }
                }

        }

        if (PATH == ""){
                bfs::path path_to_psselect (bfs::current_path());
                bfs::path path_to_psnup (bfs::current_path());
                bfs::path path_to_pdf2ps (bfs::current_path());
                path_to_psselect=bfs::system_complete( bfs::path( path_to_psselect.native_file_string()+"/psselect", bfs::native ) );
                path_to_psnup   =bfs::system_complete( bfs::path( path_to_psnup.native_file_string()+"/psnup", bfs::native ) );
                path_to_pdf2ps  =bfs::system_complete( bfs::path( path_to_pdf2ps.native_file_string()+"/pdf2ps", bfs::native ) );
                if (bfs::exists(path_to_psselect)){
                        psselect=path_to_psselect.native_file_string();
                }
                if (bfs::exists(path_to_psnup)){
                        psnup=path_to_psnup.native_file_string();
                }
                if (bfs::exists(path_to_pdf2ps)){
                        pdf2ps=path_to_pdf2ps.native_file_string();
                }
        }
        if (psselect==""){
                throw PSSELECTNOTFOUND;
        }
        if (psnup==""){
                throw PSNUPNOTFOUND;
        }

        //jetzt mal die Anzahl der Seiten feststellen
        //gleichzeitig schauen ob es ein pdf ist

        std::ifstream ifh(infile.c_str());
        std::string sline;
        char line[1000];
        bool is_dsc=false;
        while (ifh.good()){
                ifh.getline(line,999);
                sline=line;
                //Leerzeilen am Anfang berspringen
                if (sline==""){
                        continue;
                }
                //am Anfang ja "noch" nicht
                if (!is_dsc){
                        if (sline.find("%PDF-",0) != std::string::npos){
                                is_pdf=true;
                                break;
                        }
                        if (sline.find("%!PS-Adobe-",0) != std::string::npos){
                                is_dsc=true;
                        } else {
                                //kein Seitenzhlen (pagecount bleibt 0)
                                break;
                        }
                } else {
                        if (sline.find("%%Page:",0) != std::string::npos){
                                pagecount++;
                        }
                }

        }
        ifh.close();

        if (is_pdf && (pdf2ps=="")){
                        throw PDF2PSNOTFOUND;
        }

}

void PsTools::generatePostscript (std::string outfile,std::list<unsigned int> &pages)
{

	if (pages.size()==0){
		throw EMPTYPAGELIST;
	}
	std::string pagelist="";

	std::list<unsigned int>::iterator iter;
	std::list<unsigned int>::iterator eiter;

	eiter=pages.begin();
	for (iter = pages.begin(); iter != pages.end(); iter++){
		pagelist += boost::lexical_cast<std::string>(*iter);
		eiter++;
		if (eiter != pages.end()){
			pagelist +=",";
		}
	}
        std::string aufruf="";
        if (is_pdf){
	        aufruf = pdf2ps + " " + infile + " - | " + psselect + " -p" + pagelist + " | " + psnup + " -2 > " + outfile;
        } else {
	        aufruf = psselect + " -p" + pagelist + " " + infile + " | " + psnup + " -2 > " + outfile;
        }

	int exitcode = std::system (aufruf.c_str());
       	if (exitcode != 0){
        	throw UNABLETOEXECUTE;
       	}
	
}


} //end namespace pstome
